package pinball;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;

/**
 * Get the options used in the game.
 * 
 */
public class Options extends Frame implements Serializable{
	
	/* Used to save the options. */
	
	/** Done with the options. */
	private Button OK;
	
	/** Going to use a log file. */
	private Button logFileButton;
	
	/** Indicate the options have been set. */
	private transient boolean done=false;
	
	/** A thread in the system. */
	private Checkbox post;

	/** A thread in the system. */
	private Checkbox analyze;

	/** A thread in the system. */	
	private Checkbox strategy;
	
	/** Amount of pixels to move while scanning the image. */
	private TextField pixelRate;
	
	/** Image display rate. */
	private TextField frameRate;
	
	/** Amount of buffers in the system. */
	private TextField bufferSize;
	
	/** How many times to find the flippers in the system. */
	private TextField findFlippers;
	
	/** How long can a flipper stay up. */
	private TextField resetTime;	
	
	/** The amount of balls to play. Then the system exits. */
	private TextField ballsToPlay=new TextField("0");
	
	/** Name of the log file for playback. */
	private FileDialog fileDialog=new FileDialog(this,"Log File");

	
	/** The object that called the options menu. */
	private JavaQcam myParent;
	
	/**
	 * Make the options window.
	 * 
	 */
	public Options (JavaQcam myParent) {
		this.myParent=myParent;
		/* Make the window. */		
		startup();
		setVisible(true);
		System.err.println("Visible");
		toFront();		
		System.err.println("To front");
		
	}
	
	/**
	 * Create the window. 
	 */
	private void startup() {
		/* Make up the window, place the lables, buttons, check boxes, etc. . */
		
		setLayout(new GridLayout(18,1,5,0));
		setSize(300,500);
		setTitle("Game Options");
		
		
		OK=new Button();
		OK.setLabel("OK");
		OK.setBackground(Color.lightGray);
	
	
		post=new Checkbox("Post");
		analyze=new Checkbox("Analyze");
		strategy=new Checkbox("Strategy");
		
		post.setState(GameOptions.post());
		analyze.setState(GameOptions.analyze());
		strategy.setState(GameOptions.strategy());
		
		add(new Label("Which threads should run?"));
		add(post);
		add(analyze);		
		add(strategy);				

		add(new Label("Enter in the pixel rate"));
		
		/* Set the values to the values in the Game Options. */
		add(pixelRate=new TextField(new Integer(GameOptions.pixelRate()).toString(),5));

		add(new Label("Enter in the frame rate"));
		add(frameRate=new TextField(new Integer(GameOptions.frameRate()).toString(),5));

		add(new Label("Enter in the buffer size"));
		add(bufferSize=new TextField(new Integer(GameOptions.bufferSize()).toString(),5));

		add(new Label("Enter in the amount for flipper checks"));
		add(findFlippers=new TextField(new Integer(GameOptions.findFlippers()).toString(),5));

		add(new Label("Enter the time in ms that a flipper can stay up"));
		add(resetTime=new TextField(new Integer(GameOptions.resetTime()).toString(),5));
		
		add(new Label("Balls to play"));
		add(ballsToPlay);

		add(logFileButton=new Button("Log File"));
		
		add(OK);

		System.err.println("Started up");	

	}
	
	/** The events for the window. When OK is pressed, continue with the program.*/
	public boolean action(Event e, Object arg) {
			
		Object target = e.target;
		
		/* When the user presses OK setup the Game Options. */
		if(target==OK) {
		
			/* Check values for valid input. */
			int input=-1;
			
			/* Pixel Rate. */
			try {
				input=new Integer(pixelRate.getText()).intValue();
			}
			catch (Exception f){input=-1;}
			if(input<1){
				ErrorDialog error=new ErrorDialog(this,"Pixel Rate is less than 1.");		
				return true;
			}
			
			/* Frame Rate. */
			try {
				input=new Integer(frameRate.getText()).intValue();
			}
			catch (Exception f){input=-1;}
			if(input<0){
				ErrorDialog error=new ErrorDialog(this,"Frame Rate is less than 0.");		
				return true;
			}

			/* Buffer Size. */
			try {
				input=new Integer(bufferSize.getText()).intValue();
			}
			catch (Exception g){input=-1;}
			if(input<1){
				ErrorDialog error=new ErrorDialog(this,"Buffer size is less than 1.");		
				return true;
			}
						
			
			/* Find Flippers. */
			try {
				input=new Integer(findFlippers.getText()).intValue();
			}
			catch (Exception h){input=-1;}
			if(input<1){
				ErrorDialog error=new ErrorDialog(this,"Flipper tests amount is less than 1.");		
				return true;
			}
			
			/* Reset Time. */
			try {
				input=new Integer(resetTime.getText()).intValue();
			}
			catch (Exception i){input=-1;}
			if(input<1){
				ErrorDialog error=new ErrorDialog(this,"Flipper reset time is less than 1 ms.");		
				return true;
			}

			/* Balls to play. */
			try {
				input=new Integer(ballsToPlay.getText()).intValue();
			}
			catch (Exception j){input=-1;}
			if(input<0){
				ErrorDialog error=new ErrorDialog(this,"Ball to play is less than 0.");		
				return true;
			}

			
			GameOptions.post(post.getState());
			GameOptions.analyze(analyze.getState());				
			GameOptions.strategy(strategy.getState());
		
		
			GameOptions.pixelRate(new Integer(pixelRate.getText()).intValue());
			GameOptions.frameRate(new Integer(frameRate.getText()).intValue());
			GameOptions.bufferSize(new Integer(bufferSize.getText()).intValue());			
			GameOptions.findFlippers(new Integer(findFlippers.getText()).intValue());			
			GameOptions.resetTime(new Integer(resetTime.getText()).intValue());
		
			/* Set the number of balls to play in the system. */
			GameOptions.ballsToPlay(new Integer(ballsToPlay.getText()).intValue());

			String logFileName="";
			
			if(fileDialog.getFile()!=null)
				logFileName=fileDialog.getDirectory()+fileDialog.getFile();
			
			if(!logFileName.equalsIgnoreCase("")) {
				GameOptions.logFileName(logFileName);
				System.err.println("Reading from: "+logFileName);
			}
		
			fileDialog.toBack();
			fileDialog.dispose();
			done=true;
			myParent.next();
			return true;
		}
		
		if(target==logFileButton) {			
			fileDialog.show();
			return true;
		}
		return false;
	}

	
	/** 
	 * Save the information to disk. 
	 *
	 * @param s object to write to.
	 */
	private void writeObject(ObjectOutputStream s)
                                           throws IOException {
                      System.err.flush();
                      s.defaultWriteObject();                      
                      s.flush();
        }
        
	/** 
         * Read the information from the disk.
	 *
	 * @param s object to read from.
	 */
	private void readObject(ObjectInputStream s)
		throws IOException  {
			/* Set back done the "not done". */
			done=false;
                       try {s.defaultReadObject();}
  		 catch (ClassNotFoundException e){System.err.println(e);}
  		setVisible(true);
		toFront();
  	}
  	
  	/**
  	 * Save the options to disk.
  	 *
  	 * @param out the file to save to.
  	 *
  	 */
  	public void save(DataOutputStream out) {

		try {
			out.writeBoolean(GameOptions.display());
			out.writeBoolean(GameOptions.analyze());				
			out.writeBoolean(GameOptions.strategy());
		
			out.writeInt(GameOptions.pixelRate());
			out.writeInt(GameOptions.frameRate());
			out.writeInt(GameOptions.bufferSize());			
			out.writeInt(GameOptions.findFlippers());			
			out.writeInt(GameOptions.resetTime());
			out.writeInt(GameOptions.ballsToPlay());
		}
		catch (Exception e) {System.err.println(e);}
	}
	
	/**
  	 * Read the options from disk.
  	 *
  	 * @param in the file to read from.
  	 *
  	 */
	public void read(DataInputStream in) {
	
		try {
			post.setState(in.readBoolean());
			analyze.setState(in.readBoolean());		
			strategy.setState(in.readBoolean());
			
			pixelRate.setText(new Integer(in.readInt()).toString());
			frameRate.setText(new Integer(in.readInt()).toString());
			bufferSize.setText(new Integer(in.readInt()).toString());
			findFlippers.setText(new Integer(in.readInt()).toString());
			resetTime.setText(new Integer(in.readInt()).toString());
			ballsToPlay.setText(new Integer(in.readInt()).toString());
			
		}
		catch (Exception e) {System.err.println(e);}
	
	}
	
}
