package pinball;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.io.*;
import java.util.*;
import java.text.*;

/** 
 * Used to keep track of what goes on in the game.
 */
public class PinballLog {
	/** The file to log to. */
	private static PrintStream statsLog;
		
	/** Start time for the log. */
	private static long startOfGameTime=System.currentTimeMillis();
	
	/** Priority value to let through. 0 means nothing is written */
	private static int priorityLevel=0;
	
	/** Name of the strategy. */
	private static String id;
	
	/** File name to log to. */
	private static String filename;
	
	/** The ball for the file name. */
	private static int ballNumber=1;
	
	/** Start date/time the log file was made. */
	private static String dateTime;
	
	/** The header information. */
	private static Vector header=new Vector();
	
	/** The log information. */
	private static Vector information=new Vector();

	
	/** The path the store the file. */
	private static String path;
	
	/** The display status, used in the file name. */
	private static String display="d";
	
		
	/**
	 * Set the path of the log file.
	 *
	 * @param filePath the path of the file.
	 * @param video should the video be recorded as well.
	 */
	public static void setPath(String filePath, boolean video) {
		path=filePath;
		if(video)
			GameOptions.record_it();
	}
	
	/**
	 * Set the statistics file and priority.
	 *
	 * @param info the name of the stradegy.
 	 * @param priority the max level priority to go through.
	 */
	public static void setStatsFile(String info,int priority) {
		id=info;
		priorityLevel=priority;
		System.err.println("id");
		dateTime=dateTime();
		headerInfo("startedAt="+dateTime,"PinballLog");
		if(GameOptions.display()==false)
			display="nd";
		if(priorityLevel!=0) {
			makeFile();
		}
		
	}
	
	
	
	/**
	 * Set the log file for a new ball.
	 *
	 */
	public static synchronized void newBall() {
		ballNumber++;
		if(priorityLevel!=0) {			
			headerWrite();
			infoWrite();
			makeFile();
		}
	}
	
	/** Make the log file. */
	private static void makeFile() {
		/* The file name is made by taking the date&time, ball#, stratedgy name
		   and display status to make a unique file name id per ball. 
		*/
		filename=path+dateTime+"_"+ballNumber()+"_"+id+"-"+display;
		System.err.println("File name:"+filename);
		try{	
			statsLog=new PrintStream(new BufferedOutputStream(new FileOutputStream(filename)));
		}
		catch(IOException e){
			System.err.println("Statistics File Creation Error");
			System.exit(1);
		}
	}
	
	/** 
	 * Get the name of the videoLogFile.
	 */
	public static String videoLogFileName() {
		return path+dateTime+"_"+ballNumber()+"_"+id+"-"+display+".dat";
	}
	
	/** Set the ball number. */
	private static String ballNumber() {
		if(ballNumber<10)
			return new String("00"+ballNumber);
		else if(ballNumber<100)
			return new String("0"+ballNumber);
		else return new String(""+ballNumber);
	}
		
	/**
	 * Write the log info on a new line.
	 *	 
 	 * @param frame frame number.
	 * @param info The data to write to the log.
 	 * @param level The priority level for this data.
  	 * @param object caller of the method.
	 */
 	public static synchronized void writeln(int frame, String info, int level, Object called) {
		StringTokenizer st = new StringTokenizer(called.toString(),"@");
		String object=st.nextToken(); 
		

		if(level<=priorityLevel) {
			long timeNow=System.currentTimeMillis()-startOfGameTime;
			information.addElement(timeNow+":"+frame+":"+object+": "+info+"\n");
		}
	}

	/**
	 * Write the log info on the current line.
	 *	
 	 * @param frame frame number.
	 * @param info The data to write to the log.
 	 * @param level The priority level for this data.
  	 * @param object caller of the method.
	 */
	public static synchronized void write(int frame, String info, int level, Object called) {
		StringTokenizer st = new StringTokenizer(called.toString(),"@");
		String object=st.nextToken(); 

		if(level<=priorityLevel) {			
			long timeNow=System.currentTimeMillis()-startOfGameTime;
			information.addElement(timeNow+":"+frame+":"+object+": "+info);
		}
	}
	/** Create the date and time formated string.
	   The format is yyyymmdd@hhmmss
	*/
	private static String dateTime() {
		Date date=new Date();
		int month,day,year,hour,min,sec;
		
		month=date.getMonth()+1;
		day=date.getDate();
		year=date.getYear()+1900;
		
		hour=date.getHours();
		min=date.getMinutes();
		sec=date.getSeconds();
		
		String output="";
		
		output+=year;
		
		/* Display the date. */
		if(month>9)
			output+=month;
		if(month<10)
			output+="0"+month;

		if(day>9)
			output+=day;
		if(day<10)
			output+="0"+day;
				
		output+="@";
	
		/* Display the time. */		
		if(hour>9)			
			output+=hour;
		if(hour<10)
		 	output+="0"+hour;
		
		if(min>9)			
			output+=min;		
		if(min<10)
		 	output+="0"+min;
			
		if(sec>9)			
			output+=sec;		
		if(sec<10)
		 	output+="0"+sec;
		 	
 		return output;
	}
	
	/**
	 * Add info to the header.
	 *
	 * @param s String to add to the header.
   	 * @param object caller of the method.
	 */
	public static void headerInfo(String s, Object called) {
  		StringTokenizer st = new StringTokenizer(called.toString(),"@");
		String object=st.nextToken();
		
		header.addElement("HEADER:"+object+": "+s);		
	}
 	
	/** Writes the header to the disk. */
	private static synchronized void headerWrite() {
		String string;
		for(int i=0;i<header.size();i++) {
			string=(String)header.elementAt(i);
			statsLog.println(string);

		}
	}
	
	/** Writes the information to the disk. */
	private static synchronized void infoWrite() {	
		String string;		
		
		for(int i=0;i<information.size();i++) {
			string=(String)information.elementAt(i);
			statsLog.print(string);
		}
		statsLog.flush();
		statsLog.close();
		information=new Vector();
	}
	
	/**
	 * Writes the header and information of the ball currently in play.
	 * Called when exiting the system. 
	 */
	public static void closeUp() {
		if(priorityLevel!=0) {
			headerWrite();
			infoWrite();
		}
	}		
}
	
