package pinball.log;

import java.io.*;
import java.util.*;
import java.text.*;

/**
 * Find the statistics for the balls in play.
 */
public class BallStats {
	/** File that will get the data. */
	private PrintWriter output;

	/** The ball number. */
	private int ball;
	
	/** Time in play for the ball. */
	private long time;
	
	/** Number of hits for the ball. */
	private int hits;
	
	/** The balls that were in play. */
	private Vector balls;

	/**
	 * Creates the statistics for the balls in play.
	 *
	 * @param p the file to write the output to.
	 */
	public BallStats(PrintWriter p) {
		output=p;
		balls=new Vector();
	}
	
	/**
	 * Finds how long a ball was in play and the number of hits for it.
	 *
	 * @param tokens the line of tokens to process.
	 */
	public void process(StringTokenizer tokens) {
		String command;

		while(tokens.hasMoreTokens()) {
			command=tokens.nextToken();
			check(command);
		}
	}


	/** 
	 * Analyzes the ball's information. 
	 * Finds how long a ball was in play and the number of hits for it.
	 *
	 * data the line from the log file.
	 */
	private void check(String data) {
		StringTokenizer tokens;
		String command;
			
		
		tokens = new StringTokenizer(data,"=");
		
		command=tokens.nextToken();
		
		if(command.equals("Ball")) {
			ball=new Integer(tokens.nextToken()).intValue();
		}
		else if(command.equals("inPlay")) {
			time=new Long(tokens.nextToken()).longValue();
		}
		else if(command.equals("Hits")) {
			hits=new Integer(tokens.nextToken()).intValue();
			balls.addElement(new BallStat(ball,time,hits));		
		}		
	}
	
	/**
	 * Writes the information about the balls to the output file.
	 */
 	public void writeInfo() {
		BallStat ball;
		float hits=0;
		float avgHits=0;
		float time=0;
		float avgTime=0;
		
		/* Used for the size of the graphs. */
		int timeAdj=1000;
		int ballAdj=10;

		/********** TABLE. **********/
		/* Table of hits and time information. */
		output.println("<TABLE BORDER CELLPADDING=\"2\">");
		output.println("\t<TR>");
		output.println("\t\t<TD>Ball Number</TD>");
		output.println("\t\t<TD>Time in Play (ms)</TD>");
		output.println("\t\t<TD></TD>");
		output.println("\t\t<TD>Number of Hits</TD>");
		output.println("\t\t<TD></TD>");
		output.println("\t</TR>");
		
		for(int i=0;i<balls.size();i++) {
			ball=(BallStat)balls.elementAt(i);

			output.println("\t<TR>");
			output.println("\t\t<TD>"+ball.number()+"</TD>");
			output.println("\t\t<TD>"+ball.time()+"</TD>");
			output.println("\t\t<TD>"+"<IMG SRC=\"images/green.gif\" WIDTH=\""+ball.time()/timeAdj+"\" HEIGHT=\"10\"></TD>");
			output.println("\t\t<TD>"+ball.hits()+"</TD>");
			if(ball.hits()>0)
				output.print("\t\t<TD>"+"<IMG SRC=\"images/blue.gif\" WIDTH=\""+ball.hits()*ballAdj+"\" HEIGHT=\"10\">");
			else output.print("\t\t<TD>"+"<IMG SRC=\"images/blue.gif\" WIDTH=\""+1+"\" HEIGHT=\"10\">");
			output.println("</TD>");
			output.println("\t</TR>");
		}
		
		output.println("</TABLE><BR>");



		/* Find the max number of hits & time. */
		int maxHits=0;
		long maxTime=0;
		for(int i=0;i<balls.size();i++) {
			ball=(BallStat)balls.elementAt(i);
			int thisHits=ball.hits();
			long thisTime=ball.time();
			
			if(thisHits>maxHits)
				maxHits=thisHits;
				
			if(thisTime>maxTime)
				maxTime=thisTime;
		}

		int rangeHits[]=new int[maxHits+1];
		int rangeTime[]=new int[(int)(maxTime/1000)+1];
		
		/* Find the range of hits & time. */
		for(int i=0;i<balls.size();i++) {
			ball=(BallStat)balls.elementAt(i);
			/* Increment the amount of hits for this range. */
			rangeHits[ball.hits()]++;

			/* Increment the amount of time for this range. */
			rangeTime[(int)ball.time()/1000]++;

		}
		
		/********** TABLE. **********/
		/* Plot the range of hits. */
		output.println("<TABLE BORDER CELLPADDING=\"2\">");
		output.println("\t<TR>");
		output.println("\t\t<TD>Number of Hits</TD>");
		output.println("\t\t<TD>Number of Balls</TD>");
		output.println("\t\t<TD></TD>");		
		output.println("\t</TR>");

		/* Display the range of hits. */
		for(int i=0; i<rangeHits.length; i++) {
			output.println("\t<TR>");	
			output.println("\t\t<TD>"+i+"</TD>");
			output.println("\t\t<TD>"+rangeHits[i]+"</TD>");
			if(rangeHits[i]>0)
				output.print("\t\t<TD>"+"<IMG SRC=\"images/blue.gif\" WIDTH=\""+rangeHits[i]*ballAdj+"\" HEIGHT=\"10\">");
			else output.print("\t\t<TD>"+"<IMG SRC=\"images/blue.gif\" WIDTH=\""+1+"\" HEIGHT=\"10\">");				
			output.println("\t</TR>");
		}
		output.println("</TABLE><BR>");
		
		
		
		/* Table for average time and hits information. */
		output.println("<TABLE BORDER CELLPADDING=\"2\">");
		output.println("\t<TR>");
		output.println("\t\t<TD>Mean Number of Hits</TD>");
		output.println("\t\t<TD>Standard Deviation of Hits</TD>");
		output.println("\t</TR>");

		/* Get the total hits and time. */
		for(int i=0;i<balls.size();i++) {
			ball=(BallStat)balls.elementAt(i);
			time+=ball.time();
			hits+=ball.hits();
			
		}

		avgTime=time/balls.size();
		avgHits=hits/balls.size();
	
		/* Find the standard deviation. */
		double differences=0;
						
		for(int i=0;i<balls.size();i++) {
			ball=(BallStat)balls.elementAt(i);
			differences+=(avgHits-ball.hits())*(avgHits-ball.hits());
		}
		differences/=balls.size();
		double standardDeviation=Math.sqrt(differences);

		DecimalFormat mydf = new DecimalFormat( "###0.000" );
		mydf.setMinimumIntegerDigits(3);

		output.println("\t<TR>");
		output.println("\t\t<TD>"+mydf.format(avgHits)+"</TD>");		
		output.println("\t\t<TD>"+mydf.format(standardDeviation)+"</TD>");
		output.println("\t</TR>");
		
		output.println("</TABLE><BR>");			
		
		
		
		/********** TABLE. **********/		
		/* Plot the range of time. */
		output.println("<TABLE BORDER CELLPADDING=\"2\">");
		output.println("\t<TR>");
		output.println("\t\t<TD>Number of Seconds</TD>");
		output.println("\t\t<TD>Number of Balls</TD>");
		output.println("\t\t<TD></TD>");		
		output.println("\t</TR>");

		/* The gaps of time to display. */
		int timeGap=rangeTime.length/10;

		/* Display the range of time. */
		for(int i=0; i<rangeTime.length; i+=timeGap) {
			output.println("\t<TR>");	
			output.println("\t\t<TD>"+i+" - "+(i+timeGap-1)+"</TD>");

			/* Total balls for this time slot. */
			int totalBallsTimeSlot=0;
			for(int j=0; j<timeGap; j++) {
				if(i+j<rangeTime.length) {
					totalBallsTimeSlot+=rangeTime[i+j];
				}
			}
				
			output.println("\t\t<TD>"+totalBallsTimeSlot+"</TD>");
			if(totalBallsTimeSlot>0)
				output.print("\t\t<TD>"+"<IMG SRC=\"images/blue.gif\" WIDTH=\""+totalBallsTimeSlot*ballAdj+"\" HEIGHT=\"10\">");
			else output.print("\t\t<TD>"+"<IMG SRC=\"images/blue.gif\" WIDTH=\""+1+"\" HEIGHT=\"10\">");				
			output.println("\t</TR>");
		}
		output.println("</TABLE><BR>");		


		/********** TABLE. **********/
		/* Plot the averages. */
		output.println("<TABLE BORDER CELLPADDING=\"2\">");
		output.println("\t<TR>");
		output.println("\t\t<TD>Mean Time in Play (ms)</TD>");
		output.println("\t\t<TD>Standard Deviation of Time (ms)</TD>");
		
		/* Find the standard deviation. */
		differences=0;
						
		for(int i=0;i<balls.size();i++) {
			ball=(BallStat)balls.elementAt(i);
			differences+=(avgTime-ball.time())*(avgTime-ball.time());
		}
		differences/=balls.size();
		standardDeviation=Math.sqrt(differences);
								
		output.println("\t<TR>");
		output.println("\t\t<TD>"+avgTime+"</TD>");
		output.println("\t\t<TD>"+mydf.format(standardDeviation)+"</TD>");		
		output.println("\t</TR>");
		output.println("</TABLE><BR>");				

	}
}
