package pinball.log;

import pinball.*;
import pinball.buffer.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;

/**
 * Displays the video of the log file. 
 * 
 */
public class Viewer extends Frame{
	/** Canvas to draw to. */
	private ViewerCanvas canvas;
	
	/** Frame to display the video. */
	private CameraFrame frame;

	/** Width of the viewer. */
	private final int width=400;

	/** Height of the viewer. */
	private final int height=400;
	
	/** Input file. */
        private ObjectInputStream imageFile;
	
	/** Video log file. */
	private VideoLog videoLog;
	
	/** The button that starts the viewer. */
	private Button go=new Button("GO");
	
	/** Ball's location. */
	private int x=-1, y=-1;
	
	/** Reads the statistics log files. */
	private LogReport log;

	/** Is the system running. */
	private boolean running=false;
	
	/**
	 * Sets up the video viewer.
	 *
	 * @param log The file analyzer.
	 *
	 */
	public Viewer(LogReport log) {
			
		this.log=log;
		
		videoLog=new VideoLog();	

		startup();
		setVisible(true);
		toFront();
		canvas.makeImage();
	}
		
	/** Sets up the window. */
	private void startup() {
		setSize(width,height);
		setLayout(new FlowLayout(FlowLayout.CENTER));
		canvas=new ViewerCanvas(width,height/2);
		add(canvas);
		add(go);
		setTitle("Game Viewer");
	}

	/**
	 * Read a frame from the log file. 
	 *
	 * @param x the x location of the ball.
	 * @param y the y location of the ball.	 
	 */ 
	public void readFrame(int x, int y) {
		boolean hasData=true;
		CameraFrame frame=null;
		
		/* Read a frame from the log file and display it on the screen. */
		try{frame=(CameraFrame)imageFile.readObject();}
		catch(IOException e){hasData=false;}
		catch (ClassNotFoundException e){hasData=false;}			
		if(hasData)
			canvas.updateFrame(frame, x, y);		
		
	}
	
	/** 
	 * Get the next ball's video clip.
	 *
	 * @param clip the file name of the next video clip.
	 */
	public void nextBallClip(String clip) {
		System.err.println("Opening video clip: "+clip);
		try{
		imageFile=videoLog.getReadNext(clip);
		}
		catch(Exception e){System.err.println(e);}
	}
	
	public void update(Graphics g) {
		paint(g);
	}

	public void paint(Graphics g) {		
	}
	
	/**
	 * The events for the window. 
	 * 
	 * @return Was the action handled.
	 */
	public boolean action(Event e, Object arg) {
		Object target = e.target;
		
		if(target==go && !running) {
			log.process();		
			running=true;
			return true;	
		}
		
		else return false;
	}


}
