package pinball.player;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;


/**
 * The image grabber. Takes images from the camera.
 */
public class FrameGrabber extends Thread {
	/** The camera. */
	protected Camera Qcam;
	
	/** The current frame number. */
	protected int frameNumber=1;
	
	/** The current infomation buffer. */
	protected Buffer buffer;
	
	/** The contents of the buffer. */
	protected BufferInfo buffinfo;
	
	/** Did the resize of the camera take place. */
	protected boolean resized=false;
	
	/** Amount of buffers adjusted. */
	protected int edited=0;
	
	/** Image location. */
	protected int X,Y,XOffset,YOffset;
	
	/** Is the image to be resized. */
	protected boolean resizing=false;
		

	/**
	 * Make a new frame grabber thread.
	 *
	 * @param Qcam camera
	 * @param buffer first buffer in the series of buffers.
	 */
	public FrameGrabber(Camera Qcam, Buffer buffer) {
 
		/* The camera object. This is where you get the live images.*/
		this.Qcam=Qcam;
		
		/* The first buffer to store the information in. */
		this.buffer=buffer; 	
	}
	
	/**
	 * For extended classes.
	 */
	public FrameGrabber() {}
	
	/**
	 * Get the name of the object.
	 * 
	 * @param name of the object.
	 */
	public String toString() {
		return "FrameGrabber";
	}
	
	/**
	 * Grab images from the camera. 
	 */
	public void run() {
			
		buffinfo=buffer.info();

		/* First frame grabbed. */
		buffer.lock().grab(buffer.id());
		buffinfo.time().setFirstGrab();
		buffinfo.time().setStartGrabTime();


		/***********************************/	
		/* Take a picture from the camera. */
		/***********************************/	
								
  		Qcam.grabFrame(buffinfo);

		if(GameOptions.debug()>=2)
	        	System.err.println("Captured frame #"+frameNumber);    		
	     
		buffinfo.number(frameNumber);
    	          buffinfo.time().setEndGrabTime();
    	        
		frameNumber++;
		
		/* Image size, brightness, and contrast. */
		PinballLog.headerInfo("imageX="+Qcam.xSize()+" imageY="+Qcam.ySize(),this);
		PinballLog.headerInfo("brightness="+Qcam.brightness(),this);
		PinballLog.headerInfo("contrast="+Qcam.contrast(),this);

				 
       		buffer.lock().grabed(buffer.id());				        	
       		
	
		buffer=buffer.next();
	
		
		Thread.yield();
		
		/***********************************/
		/* The rest of the frames grabbed. */
		/***********************************/

		while(true) {		
	    		buffer.lock().grab(buffer.id());

		
			buffinfo=buffer.info();
			
		
			buffinfo.time().setStartGrabTime();
		
			/*************************/
			/*    SET UP RESIZE. 	*/
			/***********************/
			/* If the buffers are to be resized. Go through each of the buffers and resize them. */
			if(buffinfo.resize() || resizing) {
	
				if(edited<=GameOptions.bufferSize()) {			
					resizeImage();
				}
				else resizing=false;
			}							
		
	   		Qcam.grabFrame(buffinfo);

			if(GameOptions.debug()>=2)
	        		System.err.println("Captured frame #"+frameNumber);    		
	     
			buffinfo.number(frameNumber);
    		        buffinfo.time().setEndGrabTime();
    	        	
			buffer.skipped(false);

			frameNumber++;		
		
	       		buffer.lock().grabed(buffer.id());				        		
       				
			buffer=buffer.next();
					

			Thread.yield();
       		}    	
       	}	
	
	/** 
	 * Check to see if resize of the images is needed.
	 */
	protected void resizeImage() {
			/* Resize the images, if not already done. */
			if(!resized) {
				resize();								
				resized=true;
			}
			
			/* get the new dimensions.*/
			buffinfo.width(Qcam.xSize());
			buffinfo.height(Qcam.ySize());
			buffinfo.offsets(Qcam.XOffset(),Qcam.YOffset());
			buffinfo.cameraInfo(Qcam.xSize()*Qcam.ySize());
			
			/* Set the resize flag to resized. */
			buffinfo.resized();	
			
			/* Buffer needs to clear the display when drawn. */					
			buffinfo.clearing(true);
			
			/* Increment the amount of buffered adjusted.*/
			edited++;
			
			/* If done with all the buffers. */
			if(edited>=GameOptions.bufferSize()) {
				edited=0;
				resized=false;
				resizing=false;
				PinballLog.writeln(buffinfo.number(),"imageX="+Qcam.xSize()+" imageY="+Qcam.ySize(),1,this);
			}			
	}
	
	/**
	 * Setup the resize. 
	 */
	protected void resize() {			 
		if(buffinfo.resize()==true) {
			System.err.println("Resize started with frame: "+buffinfo.number());
			X=buffinfo.resizeX();
			Y=buffinfo.resizeY();
			XOffset=buffinfo.resizeXOffset();
			YOffset=buffinfo.resizeYOffset();
		}
	
		/* Resize this frame. */
		Qcam.resizeFrame(buffinfo);
		resizing=true;
					
	}
}
