package pinball.player.constantpartial;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;

import java.awt.*;

/**
 * Player which constantly flips the flippers only when the ball is in the frame.  
 */
public class ConstantPartialStrategy extends Strategy {
	/** Time to rest between flips. */
	private int reset= GameOptions.resetTime();
	
	/** Did flip take place. */
	private boolean flip=false;		
	
	/** For extended classed. */
	public ConstantPartialStrategy() {
	}
	
	/** 
	 * The strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
  	 * @param camera the camera for the system.
	 */
	public ConstantPartialStrategy(Buffer buffer, TextArea gameStats, Camera camera) {
		this.buffer=buffer;
		id="ConstantPartial";

		this.stub=new Stub(buffer,gameStats,this,camera);
		stub.setPriority(Thread.NORM_PRIORITY);
		
		/* Make a larger boarder around the flippers.
		   This helps the system locate the ball out of play.
		*/
		boarder=2;
		stub.boarder(boarder);		

	}
		
	
	/**
	 * Should you flip or not. This always flips.
	 */
	public void decision() {
	
		/* Set the default that it did not flip. */	
		flipperFlipped(false);
		
		/* Is there a ball to flip? If so flip it.*/
		/* Add the offset value for the ball's location so it's
		correctly found from original location in game. */
		
		x=buffinfo.blip().x();
		y=buffinfo.blip().y();
					
		if(x!=-1)	
			x+=buffinfo.XOffset();
		if(y!=-1)	
			y+=buffinfo.YOffset();
			
		/* Flip the flippers. */
		if(x>=0 && y>=flipperHeight) {
		
			stub.leftFlipperState(true);
			stub.rightFlipperState(true);
		}
			
	}			
}
