package pinball.player.remote;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import java.net.*;

/**
 * Used to listen to commands from the server. Builds up the BufferInfo and 
 * other data sent over the network.
 */
public class RemoteListener extends Thread{
	/** Client side of the network. */
	protected MyClient client;
	
	/** Stub for the strategy. */
	protected RemoteStub stub;
	
	/**
	 * Create a new remote listener.
	 *
	 * @param stub stub of strategy.
	 * @param client client site of network to read from.
	 */
	public RemoteListener(RemoteStub stub, MyClient client) {
		this.stub=stub;
		this.client=client;
	}

	/**
	 * Keep listening for commands and follow them out.
	 */
	public void run() {
		System.err.println("Remote Player is online...");
		String data;
		StringTokenizer tokens;
		String command;
		
		stub.ping();
		
		int i=1;
		
		/* Read the data from the server. */
		while((data=client.receive())!=null) {
			command=null; 			
			tokens = new StringTokenizer(data);
		
			/* Get the command and the data. */
			if(tokens.countTokens()>1)
				command=tokens.nextToken();
			else command=data;
			
//			System.err.println("Remote listing "+i);
			i++;
			
			/* Every 50 reads, do a ping. */
			if(i%50==0)
				stub.ping();
				
			/* Check the command. */
			if(command!=null) {
//				System.err.println(data);
				if(command.equals("playStatus"))
					stub.playStatus(tokens);
				else if(command.equalsIgnoreCase("buffInfo"))
					stub.buffInfo(tokens);

				else if(command.equalsIgnoreCase("buildBufferInfo"))					
					stub.buildBufferInfo();


				else if(command.equals("resetTime"))
					stub.resetTime(tokens);
				else if(command.equalsIgnoreCase("leftFlipperLocation"))
					stub.leftFlipperLocation(tokens);
				else if(command.equalsIgnoreCase("rightFlipperLocation"))
					stub.rightFlipperLocation(tokens);
				else if(command.equalsIgnoreCase("outZone"))
					stub.outZone(tokens);		
				else if(command.equalsIgnoreCase("flipperZone"))
					stub.flipperZone(tokens);
				else if(command.equalsIgnoreCase("height")) 
					stub.height(tokens);									
				else if(command.equalsIgnoreCase("blipLocation"))
					stub.blipLocation(tokens);
				
				else if(command.equals("flipperSpace"))
					stub.flipperSpace();
				else if(command.equals("topBottom"))
					stub.topBottom();
					
				else if(command.equals("foundFlippers"))
					GameOptions.foundFlippers(true);		
					
				else if(command.equals("frame"))
					stub.setImage(data);
					
				else if(command.equals("PONG"))
					stub.pong(tokens);

				/* Check with extended players for any new commands. */					
				else if(processCommand(command))
					;	
					
				/* Invalid command. */	
				else System.err.println("BAD COMMAND: "+command);	
			}
			
			/* Let another thread go. */
			Thread.yield();
		}
		System.err.println("Remote Player Offline");
		System.exit(0);
	  }
		
	/**
	 * Process the command from the network. Must return true or false to
	 * indicate if the command was processed.
	 *
	 * @param command the command from the network.
	 * @return the indication of weither the command was processed.
	 */
	 public boolean processCommand(String command) {
	 	return false;	 
	 }
}
